// LogoBlendImpl.h : Declaration of the CLogoBlendImpl

#pragma once
#include "resource.h"
#include "ULogoBlend.h"

// Helper class for automatic HANDLE release.
class CAutoHandle
{
public:

	CAutoHandle() : m_hHandle(INVALID_HANDLE_VALUE)
	{
	}

	CAutoHandle(HANDLE hHandle) : m_hHandle(hHandle)
	{
	}

	~CAutoHandle()
	{
		if (m_hHandle != INVALID_HANDLE_VALUE)
		{
			CloseHandle(m_hHandle);
			m_hHandle = INVALID_HANDLE_VALUE;
		}
	}

	operator HANDLE()
	{
		return m_hHandle;
	}	

private:
	HANDLE m_hHandle;
};


// CLogoBlendImpl class declaration.
class ATL_NO_VTABLE CLogoBlendImpl : 
	public CComObjectRootEx<CComMultiThreadModel>,
	public CComCoClass<CLogoBlendImpl, &CLSID_LogoBlend>,
	public IUVideoTransformer // The component must implement this interface to work with ULiveServer.
{
public:
	CLogoBlendImpl() : m_pLogoBits(NULL)
	{
		ZeroMemory(&m_LogoBMPHeader, sizeof(BITMAPINFOHEADER));
	}

	~CLogoBlendImpl()
	{
		if (m_pLogoBits)
		{
			delete [] m_pLogoBits;
			m_pLogoBits = NULL;
		}
	}

DECLARE_REGISTRY_RESOURCEID(IDR_LOGOBLENDIMPL)


BEGIN_COM_MAP(CLogoBlendImpl)	
	COM_INTERFACE_ENTRY(IUnknown)
	COM_INTERFACE_ENTRY(IUVideoTransformer)
END_COM_MAP()


	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		HRESULT hr = S_OK;

		try
		{			
			if (!LoadLogo(_T("Logo.bmp")))
			{
				hr = E_FAIL; // Better error reporting can be used.
			}
		}
		catch (...)
		{
			hr = E_UNEXPECTED;
		}

		return hr;
	}
	
public:
// IUVideoTransformer interface methods.
	STDMETHOD(Transform)(long lResourceID, U_VideoFrame *pInFrame, U_VideoFrameData **pOutFrame);

// Class methods.
private:
	bool LoadLogo(LPCTSTR szLogoFileName);
	bool TransparentBlend(BYTE *pDestBytes, BITMAPINFOHEADER &Destbmh, 
					      BYTE *pSourceBytes, BITMAPINFOHEADER &Sourcebmh, 
						  long nLogoWeight);
	bool LoadBitmapFile(LPCTSTR szFileName, BITMAPINFOHEADER *pBmh, PBYTE *ppBuffer);

// Class data.
private:
	BITMAPINFOHEADER m_LogoBMPHeader;
	PBYTE            m_pLogoBits;
};

OBJECT_ENTRY_AUTO(__uuidof(LogoBlend), CLogoBlendImpl)
